"""

gists
~~~~~

:copyright: (c) 2012 by Jaume Devesa
:license: MIT, see LICENSE for more details

"""

__title__ = 'gists'
__version__ = '0.1'
__author__ = 'Jaume Devesa'
__license__ = 'MIT'
__copyright__ = 'Copyright 2012 Jaume Devesa'
